@@ -5,7 +5,10 @@
 
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import padding
-from cryptography.hazmat.primitives.ciphers import CipherAlgorithm
+from cryptography.hazmat.primitives.ciphers import (
+    BlockCipherAlgorithm,
+    CipherAlgorithm,
+)
 from cryptography.hazmat.primitives.ciphers.modes import Mode
 
 
@@ -16,6 +19,13 @@ class DummyCipherAlgorithm(CipherAlgorithm):
     key_sizes = frozenset([256])
 
 
+class DummyBlockCipherAlgorithm(DummyCipherAlgorithm, BlockCipherAlgorithm):
+    def __init__(self, _: object) -> None:
+        pass
+
+    name = "dummy-block-cipher"
+
+
 class DummyMode(Mode):
     name = "dummy-mode"
 
