@@ -8,19 +8,15 @@ import os
 
 import pytest
 
+from cryptography.exceptions import AlreadyFinalized, _Reasons
+from cryptography.hazmat.bindings._rust import openssl as rust_openssl
 from cryptography.hazmat.primitives.ciphers import algorithms, base, modes
 
 from ...doubles import DummyMode
-from ...utils import load_nist_vectors
+from ...utils import load_nist_vectors, raises_unsupported_algorithm
 from .utils import _load_all_params, generate_encrypt_test
 
 
-@pytest.mark.supported(
-    only_if=lambda backend: backend.cipher_supported(
-        algorithms.AES(b"\x00" * 32), modes.XTS(b"\x00" * 16)
-    ),
-    skip_message="Does not support AES XTS",
-)
 class TestAESModeXTS:
     def test_xts_vectors(self, backend, subtests):
         # This list comprehension excludes any vector that does not have a
@@ -42,9 +38,11 @@ class TestAESModeXTS:
                 tweak = binascii.unhexlify(vector["i"])
                 pt = binascii.unhexlify(vector["pt"])
                 ct = binascii.unhexlify(vector["ct"])
-                cipher = base.Cipher(
-                    algorithms.AES(key), modes.XTS(tweak), backend
-                )
+                alg = algorithms.AES(key)
+                mode = modes.XTS(tweak)
+                if not backend.cipher_supported(alg, mode):
+                    pytest.skip(f"AES-{alg.key_size}-XTS not supported")
+                cipher = base.Cipher(alg, mode, backend)
                 enc = cipher.encryptor()
                 computed_ct = enc.update(pt) + enc.finalize()
                 assert computed_ct == ct
@@ -52,24 +50,38 @@ class TestAESModeXTS:
                 computed_pt = dec.update(ct) + dec.finalize()
                 assert computed_pt == pt
 
-    def test_xts_too_short(self, backend):
-        key = b"thirty_two_byte_keys_are_great!!"
-        tweak = b"\x00" * 16
-        cipher = base.Cipher(algorithms.AES(key), modes.XTS(tweak))
-        enc = cipher.encryptor()
-        with pytest.raises(ValueError):
-            enc.update(b"0" * 15)
+    def test_xts_too_short(self, backend, subtests):
+        for key in [
+            b"thirty_two_byte_keys_are_great!!",
+            b"\x00" * 32 + b"\x01" * 32,
+        ]:
+            with subtests.test():
+                key = b"\x00" * 32 + b"\x01" * 32
+                mode = modes.XTS(b"\x00" * 16)
+                alg = algorithms.AES(key)
+                if not backend.cipher_supported(alg, mode):
+                    pytest.skip(f"AES-{alg.key_size}-XTS not supported")
+                cipher = base.Cipher(alg, mode)
+                enc = cipher.encryptor()
+                with pytest.raises(ValueError):
+                    enc.update(b"0" * 15)
 
     @pytest.mark.supported(
-        only_if=lambda backend: (not backend._lib.CRYPTOGRAPHY_IS_LIBRESSL),
+        only_if=lambda backend: not rust_openssl.CRYPTOGRAPHY_IS_LIBRESSL,
         skip_message="duplicate key encryption error added in OpenSSL 1.1.1d",
     )
-    def test_xts_no_duplicate_keys_encryption(self, backend):
-        key = bytes(range(16)) * 2
-        tweak = b"\x00" * 16
-        cipher = base.Cipher(algorithms.AES(key), modes.XTS(tweak))
-        with pytest.raises(ValueError, match="duplicated keys"):
-            cipher.encryptor()
+    def test_xts_no_duplicate_keys_encryption(self, backend, subtests):
+        key1 = bytes(range(16)) * 2
+        key2 = key1 + key1
+        mode = modes.XTS(b"\x00" * 16)
+        for key in [key1, key2]:
+            with subtests.test():
+                alg = algorithms.AES(key)
+                cipher = base.Cipher(alg, mode)
+                if not backend.cipher_supported(alg, mode):
+                    pytest.skip(f"AES-{alg.key_size}-XTS not supported")
+                with pytest.raises(ValueError, match="duplicated keys"):
+                    cipher.encryptor()
 
     def test_xts_unsupported_with_aes128_aes256_classes(self):
         with pytest.raises(TypeError):
@@ -272,7 +284,7 @@ def test_buffer_protocol_alternate_modes(mode, backend):
     data = bytearray(b"sixteen_byte_msg")
     key = algorithms.AES(bytearray(os.urandom(32)))
     if not backend.cipher_supported(key, mode):
-        pytest.skip(f"AES in {mode.name} mode not supported")
+        pytest.skip(f"AES-{key.key_size} in {mode.name} mode not supported")
     cipher = base.Cipher(key, mode, backend)
     enc = cipher.encryptor()
     ct = enc.update(data) + enc.finalize()
@@ -304,3 +316,61 @@ def test_alternate_aes_classes(mode, alg_cls, backend):
     dec = cipher.decryptor()
     pt = dec.update(ct) + dec.finalize()
     assert pt == data
+
+
+def test_reset_nonce(backend):
+    data = b"helloworld" * 10
+    nonce = b"\x00" * 16
+    nonce_alt = b"\xee" * 16
+    cipher = base.Cipher(
+        algorithms.AES(b"\x00" * 16),
+        modes.CTR(nonce),
+    )
+    cipher_alt = base.Cipher(
+        algorithms.AES(b"\x00" * 16),
+        modes.CTR(nonce_alt),
+    )
+    enc = cipher.encryptor()
+    ct1 = enc.update(data)
+    assert len(ct1) == len(data)
+    for _ in range(2):
+        enc.reset_nonce(nonce)
+        assert enc.update(data) == ct1
+    # Reset the nonce to a different value
+    # and check it matches with a different context
+    enc_alt = cipher_alt.encryptor()
+    ct2 = enc_alt.update(data)
+    enc.reset_nonce(nonce_alt)
+    assert enc.update(data) == ct2
+    enc_alt.finalize()
+    enc.finalize()
+    with pytest.raises(AlreadyFinalized):
+        enc.reset_nonce(nonce)
+    dec = cipher.decryptor()
+    assert dec.update(ct1) == data
+    for _ in range(2):
+        dec.reset_nonce(nonce)
+        assert dec.update(ct1) == data
+    # Reset the nonce to a different value
+    # and check it matches with a different context
+    dec_alt = cipher_alt.decryptor()
+    dec.reset_nonce(nonce_alt)
+    assert dec.update(ct2) == dec_alt.update(ct2)
+    dec_alt.finalize()
+    dec.finalize()
+    with pytest.raises(AlreadyFinalized):
+        dec.reset_nonce(nonce)
+
+
+def test_reset_nonce_invalid_mode(backend):
+    iv = b"\x00" * 16
+    c = base.Cipher(
+        algorithms.AES(b"\x00" * 16),
+        modes.CBC(iv),
+    )
+    enc = c.encryptor()
+    with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_CIPHER):
+        enc.reset_nonce(iv)
+    dec = c.decryptor()
+    with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_CIPHER):
+        dec.reset_nonce(iv)
