@@ -10,12 +10,7 @@ import os
 
 import pytest
 
-from cryptography.exceptions import (
-    InvalidSignature,
-    UnsupportedAlgorithm,
-    _Reasons,
-)
-from cryptography.hazmat.bindings._rust import openssl as rust_openssl
+from cryptography.exceptions import InvalidSignature, _Reasons
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import padding, rsa
 from cryptography.hazmat.primitives.asymmetric import utils as asym_utils
@@ -251,10 +246,6 @@ class TestRSA:
         assert public_num.n == public_num2.n
         assert public_num.e == public_num2.e
 
-    @pytest.mark.supported(
-        only_if=lambda backend: not rust_openssl.CRYPTOGRAPHY_IS_BORINGSSL,
-        skip_message="Does not support RSA PSS loading",
-    )
     @pytest.mark.parametrize(
         "path",
         [
@@ -302,24 +293,6 @@ class TestRSA:
                 b"badsig", b"whatever", padding.PKCS1v15(), hashes.SHA256()
             )
 
-    @pytest.mark.supported(
-        only_if=lambda backend: rust_openssl.CRYPTOGRAPHY_IS_BORINGSSL,
-        skip_message="Test requires a backend without RSA-PSS key support",
-    )
-    def test_load_pss_unsupported(self, backend):
-        # Key loading errors unfortunately have multiple paths so
-        # we need to allow ValueError and UnsupportedAlgorithm
-        with pytest.raises((UnsupportedAlgorithm, ValueError)):
-            load_vectors_from_file(
-                filename=os.path.join(
-                    "asymmetric", "PKCS8", "rsa_pss_2048.pem"
-                ),
-                loader=lambda p: serialization.load_pem_private_key(
-                    p.read(), password=None
-                ),
-                mode="rb",
-            )
-
     @pytest.mark.parametrize(
         "vector",
         load_vectors_from_file(
@@ -530,11 +503,20 @@ class TestRSASignature:
                     hashes.SHA1(),
                 )
 
+    @pytest.mark.supported(
+        only_if=lambda backend: backend.rsa_padding_supported(
+            padding.PSS(
+                mgf=padding.MGF1(hashes.SHA256()),
+                salt_length=padding.PSS.MAX_LENGTH,
+            )
+        ),
+        skip_message="Does not support PSS with these parameters.",
+    )
     @pytest.mark.parametrize(
         "hash_alg",
         [hashes.SHA224(), hashes.SHA256(), hashes.SHA384(), hashes.SHA512()],
     )
-    def test_pss_signing_sha2(self, rsa_key_2048, hash_alg, backend):
+    def test_pss_sha2_max_length(self, rsa_key_2048, hash_alg, backend):
         _skip_pss_hash_algorithm_unsupported(backend, hash_alg)
         private_key = rsa_key_2048
         public_key = private_key.public_key()
@@ -821,8 +803,8 @@ class TestRSASignature:
         ),
         skip_message="Does not support PSS.",
     )
-    def test_unsupported_hash(self, rsa_key_512: rsa.RSAPrivateKey, backend):
-        private_key = rsa_key_512
+    def test_unsupported_hash(self, rsa_key_2048: rsa.RSAPrivateKey, backend):
+        private_key = rsa_key_2048
         message = b"one little message"
         pss = padding.PSS(mgf=padding.MGF1(hashes.SHA256()), salt_length=0)
         with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_HASH):
@@ -850,9 +832,9 @@ class TestRSASignature:
         skip_message="Does not support PSS.",
     )
     def test_prehashed_digest_mismatch(
-        self, rsa_key_512: rsa.RSAPrivateKey, backend
+        self, rsa_key_2048: rsa.RSAPrivateKey, backend
     ):
-        private_key = rsa_key_512
+        private_key = rsa_key_2048
         message = b"one little message"
         h = hashes.Hash(hashes.SHA512(), backend)
         h.update(message)
@@ -1067,7 +1049,7 @@ class TestRSAVerification:
                 salt_length=padding.PSS.AUTO,
             )
         ),
-        skip_message="Does not support PSS.",
+        skip_message="Does not support PSS with these parameters.",
     )
     def test_pss_verify_auto_salt_length(
         self, rsa_key_2048: rsa.RSAPrivateKey, backend
@@ -1207,7 +1189,7 @@ class TestRSAVerification:
         public_key = private_key.public_key()
         pss_padding = padding.PSS(
             mgf=padding.MGF1(algorithm=hashes.SHA256()),
-            salt_length=padding.PSS.MAX_LENGTH,
+            salt_length=padding.PSS.DIGEST_LENGTH,
         )
         signature = private_key.sign(b"sign me", pss_padding, hashes.SHA256())
 
@@ -2137,6 +2119,8 @@ class TestRSAEncryption:
         skip_message="Requires FIPS",
     )
     def test_rsa_fips_small_key(self, rsa_key_512: rsa.RSAPrivateKey, backend):
+        # Ideally this would use a larger disallowed key like RSA-1024, but
+        # RHEL-8 thinks that RSA-1024 is allowed by FIPS.
         with pytest.raises(ValueError):
             rsa_key_512.sign(b"somedata", padding.PKCS1v15(), hashes.SHA512())
 
@@ -2396,6 +2380,12 @@ class TestRSAPrimeFactorRecovery:
     def test_invalid_recover_prime_factors(self):
         with pytest.raises(ValueError):
             rsa.rsa_recover_prime_factors(34, 3, 7)
+        with pytest.raises(ValueError):
+            rsa.rsa_recover_prime_factors(629, 17, 20)
+        with pytest.raises(ValueError):
+            rsa.rsa_recover_prime_factors(21, 1, 1)
+        with pytest.raises(ValueError):
+            rsa.rsa_recover_prime_factors(21, -1, -1)
 
 
 class TestRSAPrivateKeySerialization:
@@ -2777,3 +2767,9 @@ class TestRSAPEMPublicKeySerialization:
         key2 = copy.copy(key1)
 
         assert key1 == key2
+
+    def test_private_key_copy(self, rsa_key_2048: rsa.RSAPrivateKey):
+        key1 = rsa_key_2048
+        key2 = copy.copy(key1)
+
+        assert key1 == key2
